# -*- coding: utf-8 -*-
# Copyright (C) Brian Moe, Branson Stephens (2015)
#
# This file is part of gracedb
#
# gracedb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gracedb.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function
import unittest
import os
from .base import TestGraceDb
from xml.etree import ElementTree as ET


class TestVOEvents(TestGraceDb):
    """Integration tests for VOEvents"""

    @classmethod
    def setUpClass(cls):
        super(TestVOEvents, cls).setUpClass()

        # Create event and get its graceid
        eventFile = os.path.join(cls.TEST_DATA_DIR, "cbc-lm.xml")
        r = cls._gracedb.createEvent("Test", "gstlal", eventFile, "LowMass")
        event = r.json()
        cls._graceid = event['graceid']

        # Upload fake skymap file to use later
        r = cls._gracedb.writeLog(cls._graceid, "Fake skymap file.",
            filename="fake_skymap.txt", filecontents="Fake skymap.",
            tagname="sky_loc")

    # Helper functions --------------------------------------------------------
    def get_citations_dict(self, graceid, voevent_filename):
        """Gets a dictionary of ivorns and citation types"""

        # Get voevent file
        voevent_file_text = self._gracedb.files(graceid,
            voevent_filename).read()

        # Parse XML
        voevent_xml = ET.fromstring(voevent_file_text)
        citations_dict = {}
        for citations in voevent_xml.iterfind('Citations'):
            for e in citations.iterfind('EventIVORN'):
                ivorn = e.text
                citations_dict[ivorn] = e.attrib['cite']
        return citations_dict

    def get_ivorn(self, graceid, voevent_filename):
        """Extracts ivorn"""
        # Get voevent file
        voevent_file_text = self._gracedb.files(graceid,
            voevent_filename).read()
        return ET.fromstring(voevent_file_text).get('ivorn')

    # Tests -------------------------------------------------------------------
    def test_create_preliminary_voevent(self):
        """Create a preliminary VOEvent"""
        r = self._gracedb.createVOEvent(self._graceid, "Preliminary")
        rdict = r.json()
        self.assertTrue('voevent_type' in list(rdict.keys()))

    def test_retrieve_preliminary_voevent(self):
        """Retrieve preliminary VOEvent"""
        r = self._gracedb.voevents(self._graceid)
        voevent_list = r.json()['voevents']
        self.assertTrue(len(voevent_list) == 1 and
            voevent_list[0]['voevent_type'] == 'PR')

    def test_create_update_voevent(self):
        """Create an update VOEvent"""
        r = self._gracedb.createVOEvent(self._graceid, "Update",
            skymap_filename="fake_skymap.txt", skymap_type="FAKE",
            ProbHasRemnant=0.2, BBH=0.1, Terrestrial=0.3)
        rdict = r.json()
        self.assertTrue('voevent_type' in list(rdict.keys()))

    def test_ivorns_unique(self):
        """Compare ivorns for preliminary and update VOEvents"""
        r = self._gracedb.voevents(self._graceid)
        voevent_list = r.json()['voevents']
        # Make sure there are 2 voevents
        self.assertTrue(len(voevent_list) == 2)

        # Make sure the ivorns are different
        voevent_dict = {v['voevent_type']: v for v in voevent_list}
        preliminary_ivorn = self.get_ivorn(self._graceid,
            voevent_dict['PR']['filename'])
        update_ivorn = self.get_ivorn(self._graceid,
            voevent_dict['UP']['filename'])
        self.assertNotEqual(preliminary_ivorn, update_ivorn)

    def test_citation_section(self):
        """Test VOEvent citations"""
        r = self._gracedb.voevents(self._graceid)
        voevent_list = r.json()['voevents']

        voevent_dict = {v['voevent_type']: v for v in voevent_list}
        preliminary_ivorn = self.get_ivorn(self._graceid,
            voevent_dict['PR']['filename'])
        update_citations = self.get_citations_dict(self._graceid,
            voevent_dict['UP']['filename'])
        self.assertEqual(update_citations[preliminary_ivorn], 'supersedes')

    def test_create_retraction_voevent(self):
        """Create a retraction VOEvent"""
        r = self._gracedb.createVOEvent(self._graceid, "Retraction")
        rdict = r.json()
        self.assertTrue('voevent_type' in list(rdict.keys()))

    def test_retraction_citations(self):
        """Test retraction VOEvent citations"""
        r = self._gracedb.voevents(self._graceid)
        voevent_list = r.json()['voevents']
        voevent_dict = {v['voevent_type']: v for v in voevent_list}

        # Parse retraction voevent and check for correct citations
        retraction_citations = self.get_citations_dict(self._graceid,
            voevent_dict['RE']['filename'])
        preliminary_ivorn = self.get_ivorn(self._graceid,
            voevent_dict['PR']['filename'])
        update_ivorn = self.get_ivorn(self._graceid,
            voevent_dict['UP']['filename'])
        self.assertTrue(retraction_citations[preliminary_ivorn] == 'retraction'
            and retraction_citations[update_ivorn] == 'retraction')


# Define a custom test suite because there is some dependency between
# the successive individual tests; as a result, the order is important.
def VOEventTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(TestVOEvents('test_create_preliminary_voevent'))
    suite.addTest(TestVOEvents('test_retrieve_preliminary_voevent'))
    suite.addTest(TestVOEvents('test_create_update_voevent'))
    suite.addTest(TestVOEvents('test_ivorns_unique'))
    suite.addTest(TestVOEvents('test_citation_section'))
    suite.addTest(TestVOEvents('test_create_retraction_voevent'))
    suite.addTest(TestVOEvents('test_retraction_citations'))
    return suite




import os
import re
import shlex
import six
import stat
from datetime import datetime
from functools import wraps
from netrc import netrc, NetrcParseError
from subprocess import Popen, PIPE
if os.name == 'posix':
    import pwd

EVENT_PREFIXES = ['G', 'E', 'H', 'M', 'T']
event_prefix_regex = re.compile(r'^({prefixes})\d+'.format(
    prefixes="|".join(EVENT_PREFIXES)))

# Decorator for class methods so that they work for events or superevents
def event_or_superevent(func):
    @wraps(func)
    def inner(self, object_id, *args, **kwargs):
        is_superevent = True
        if event_prefix_regex.match(object_id):
            is_superevent = False
        return func(self, object_id, is_superevent=is_superevent, *args,
            **kwargs)
    return inner


# Function for checking arguments which can be strings or lists
# If a string, converts to list for ease of processing
def handle_str_or_list_arg(arg, arg_name):
    if arg:
        if isinstance(arg, six.string_types):
            arg = [arg]
        elif isinstance(arg, list):
            pass
        else:
            raise TypeError("{0} arg is {1}, should be str or list"
                .format(arg_name, type(arg)))
    return arg


# XXX It would be nice to get rid of this if we can.
# It seems that you can pass python lists via the requests package.
# That would make putting our lists into comma-separated strings
# unnecessary.
def cleanListInput(list_arg):
    stringified_list = list_arg
    if isinstance(list_arg, float) or isinstance(list_arg, int):
        stringified_value = str(list_arg)
        return stringified_value
    if not isinstance(list_arg, six.string_types):
        stringified_list = ','.join(map(str, list_arg))
    return stringified_list


# The following are used to check whether a user has tried to use
# an expired certificate.

# Parse a datetime object out of the openssl output.
# Note that this returns a naive datetime object.
def get_dt_from_openssl_output(s):
    dt = None
    err = ''
    # Openssl spits out a string like "notAfter=Feb  6 15:17:54 2016 GMT"
    # So we first have to split off the bit after the equal sign.
    try:
        date_string = s.split('=')[1].strip()
    except Exception as e:
        err = 'Openssl output not understood: {0}'.format(e)
        return dt, err

    # Next, attempt to parse the date with strptime.
    try:
        dt = datetime.strptime(date_string, "%b %d %H:%M:%S %Y %Z")
    except Exception as e:
        err = 'Could not parse time string from openssl: {0}'.format(e)
        return dt, err

    return dt, err


# Given a path to a cert file, check whether it is expired.
def is_expired(cert_file):
    cmd = 'openssl x509 -enddate -noout -in %s' % cert_file
    p = Popen(shlex.split(cmd), stdout=PIPE, stderr=PIPE)
    out, err = p.communicate()

    expired = None
    if p.returncode == 0:
        dt, err = get_dt_from_openssl_output(out)

        if dt:
            # Note that our naive datetime must be compared with a UTC
            # datetime that has been rendered naive.
            expired = dt <= datetime.utcnow().replace(tzinfo=None)

    return expired, err


class safe_netrc(netrc):
    """The netrc.netrc class from the Python standard library applies access
    safety checks (requiring that the netrc file is readable only by the
    current user, and not by group members or other users) only if using the
    netrc file in the default location (~/.netrc). This subclass applies the
    same access safety checks regardless of the path to the netrc file."""

    def _parse(self, file, fp, *args, **kwargs):
        # Copied and adapted from netrc.py from Python 2.7
        if os.name == 'posix':
            prop = os.fstat(fp.fileno())
            if prop.st_uid != os.getuid():
                try:
                    fowner = pwd.getpwuid(prop.st_uid)[0]
                except KeyError:
                    fowner = 'uid %s' % prop.st_uid
                try:
                    user = pwd.getpwuid(os.getuid())[0]
                except KeyError:
                    user = 'uid %s' % os.getuid()
                raise NetrcParseError(
                    ("~/.netrc file owner (%s) does not match"
                     " current user (%s)") % (fowner, user),
                    file)
            if (prop.st_mode & (stat.S_IRWXG | stat.S_IRWXO)):
                raise NetrcParseError(
                    "~/.netrc access too permissive: access"
                    " permissions must restrict access to only"
                    " the owner", file)
        return netrc._parse(self, file, fp, *args, **kwargs)



# -*- coding: utf-8 -*-
# Copyright (C) Brian Moe, Branson Stephens (2015)
#
# This file is part of gracedb
#
# gracedb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gracedb.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function
import unittest
import random
import os
from datetime import datetime
import time

from ligo.gracedb.rest import GraceDb
from six.moves import range

from .base import TestGraceDb


class TestEvents(TestGraceDb):
    """Integration tests for event creation and annotation"""

    @classmethod
    def setUpClass(cls):
        # Create an event for testing so we don't have to create a lot of
        # events for all of the unit tests.
        super(TestEvents, cls).setUpClass()

        # For testing of labels and offline variable
        cls._labelName = "DQV"
        cls._offline = False

        # Create event and get its graceid
        cls._eventFile = os.path.join(cls.TEST_DATA_DIR, "cbc-lm.xml")
        cls._createdEvent = cls._gracedb.createEvent("Test", "gstlal",
            cls._eventFile, "LowMass", offline=cls._offline).json()
        cls._eventId = cls._createdEvent["graceid"]

    def test_root(self):
        """Did root resource retrieval succeed?"""
        self.assertTrue("CBC" in self._gracedb.groups)
        pass

    def test_get_events(self):
        """
        Get the events resource.
        Make sure first event looks like an event.
        """
        events = self._gracedb.events()
        for event in events:
            self.assertTrue('graceid' in event)
            break

    def test_create_log(self):
        """Create an event log message"""
        message = "Message is {0}".format(random.random())
        resp = self._gracedb.writeLog(self._eventId, message)
        self.assertEqual(resp.status, 201)
        new_log_uri = resp.getheader('Location')
        new_log = resp.json()
        self.assertEqual(new_log_uri, new_log['self'])
        check_new_log = self._gracedb.get(new_log_uri).json()
        self.assertEqual(check_new_log['comment'], message)

    def test_get_log(self):
        """Retrieve event log"""
        logs = self._gracedb.logs(self._eventId).json()
        self.assertTrue('numRows' in logs)

    def test_create_emobservation(self):
        """Create an EM observation entry."""
        comment = "Message is {0}".format(random.random())
        # Let's put in some made-up values
        raList = [1.0,1.0,1.0]
        raWidthList = 1.0
        decList = [1.0,1.0,1.0]
        decWidthList = 1.0
        dt = datetime(1900,1,1,1,1,1)
        startTimeList = [dt.isoformat() for i in range(3)]
        durationList = 1.0
        resp = self._gracedb.writeEMObservation(self._eventId, 'Test',
            raList, raWidthList, decList, decWidthList,
            startTimeList, durationList, comment)
        self.assertEqual(resp.status, 201)
        new_emobservation_uri = resp.getheader('Location')
        new_emobservation = resp.json()
        self.assertEqual(new_emobservation_uri, new_emobservation['self'])
        check_new_emobservation = self._gracedb.get(new_emobservation_uri).json()
        self.assertEqual(check_new_emobservation['comment'], comment)

    def test_get_emobservations(self):
        """Retrieve EM Observation List"""
        emos = self._gracedb.emobservations(self._eventId).json()
        self.assertTrue('numRows' in emos)

    def test_upload_large_file(self):
        """
        Upload a large file.
        Issue https://bugs.ligo.org/redmine/issues/951
        """
        uploadFile = os.path.join(self.TEST_DATA_DIR, "big.data")
        r = self._gracedb.writeLog(self._eventId, "FILE UPLOAD", uploadFile)
        self.assertEqual(r.status, 201) # CREATED
        r_content = r.json()
        link = r_content['file']
        self.assertEqual(open(uploadFile, 'rb').read(),
            self._gracedb.get(self._gracedb.files(self._eventId).json()
            ['big.data']).read())
        self.assertEqual(open(uploadFile, 'rb').read(),
            self._gracedb.get(link).read())

    def test_upload_file(self):
        """Upload and re-upload a file"""

        uploadFile = os.path.join(self.TEST_DATA_DIR, "upload.data")
        r = self._gracedb.writeLog(self._eventId, "FILE UPLOAD", uploadFile)
        self.assertEqual(r.status, 201) # CREATED
        r_content = r.json()
        link = r_content['file']

        self.assertEqual(open(uploadFile, 'rb').read(),
            self._gracedb.get(self._gracedb.files(self._eventId).json()
            ['upload.data']).read())

        self.assertEqual(open(uploadFile, 'rb').read(),
            self._gracedb.get(link).read())

        # Re-upload slightly different file.
        uploadFile2 = os.path.join(self.TEST_DATA_DIR, "upload2.data")
        r = self._gracedb.writeLog(self._eventId, "FILE UPLOAD",
            filename="upload.data", filecontents=open(uploadFile2, 'r'))
        self.assertEqual(r.status, 201) # CREATED
        r_content = r.json()
        link2 = r_content['file']

        self.assertEqual(open(uploadFile2, 'rb').read(),
            self._gracedb.get(self._gracedb.files(self._eventId).json()
            ['upload.data']).read())

        self.assertEqual(open(uploadFile2, 'rb').read(),
            self._gracedb.get(link2).read())

        self.assertNotEqual(link, link2)

    def test_files(self):
        """Get file info"""
        r = self._gracedb.files(self._eventId)
        event = r.json()
        self.assertEqual(r.status, 200)
        self.assertTrue(isinstance(event, dict))

    def test_create_cwb(self):
        """Create a CWB event"""
        """burst-cwb.txt"""
        time.sleep(self.SLEEP_TIME)
        eventFile = os.path.join(self.TEST_DATA_DIR, "burst-cwb.txt")
        r = self._gracedb.createEvent("Test", "CWB", eventFile)
        self.assertEqual(r.status, 201) # CREATED
        cwb_event = r.json()
        self.assertEqual(cwb_event['group'], "Test")
        self.assertEqual(cwb_event['pipeline'], "CWB")
        self.assertEqual(float(cwb_event['gpstime']), 1042312876.5090)

    def test_create_lowmass(self):
        """Create a Low Mass event"""
        """cbc-lm.xml"""
        # This is done with the initially created event.
        pass

    def test_create_mbta(self):
        """Create an MBTA event"""
        """cbc-mbta.xml"""
        time.sleep(self.SLEEP_TIME)
        eventFile = os.path.join(self.TEST_DATA_DIR, "cbc-mbta.xml")
        mbta_event = self._gracedb.createEvent(
                "Test", "MBTAOnline", eventFile).json()
        self.assertEqual(mbta_event['group'], "Test")
        self.assertEqual(mbta_event['pipeline'], "MBTAOnline")
        self.assertEqual(float(mbta_event['gpstime']), 1078903329.421037)
        original_far = 4.006953918826065e-7
        self.assertTrue((mbta_event['far'] - original_far) < original_far*1e-14)

    def test_create_olib(self):
        """Create an oLIB event"""
        time.sleep(self.SLEEP_TIME)
        eventFile = os.path.join(self.TEST_DATA_DIR, "olib-test.json")
        event = self._gracedb.createEvent("Test", "oLIB", eventFile).json()
        self.assertEqual(event['group'], "Test")
        self.assertEqual(event['pipeline'], "oLIB")
        self.assertEqual(event['far'], 7.22e-06)
        self.assertEqual(event['extra_attributes']['LalInferenceBurst']['bci'],
            1.111)

    def test_create_spiir(self):
        """Create a spiir event"""
        time.sleep(self.SLEEP_TIME)
        eventFile = os.path.join(self.TEST_DATA_DIR, "spiir-test.xml")
        event = self._gracedb.createEvent("Test", "spiir", eventFile).json()
        self.assertEqual(event['group'], "Test")
        self.assertEqual(event['pipeline'], "spiir")
        self.assertEqual(event['far'], 3.27e-07)
        self.assertEqual(event['extra_attributes']['CoincInspiral']['mass'],
            3.98)

    def test_create_hardwareinjection(self):
        """Create a HardwareInjection event"""
        """sim-inj.xml"""
        time.sleep(self.SLEEP_TIME)
        eventFile = os.path.join(self.TEST_DATA_DIR, "sim-inj.xml")
        # Don't need to specify source_channel or destination_channel
        hardwareinjection_event = self._gracedb.createEvent("Test",
            "HardwareInjection", eventFile, instrument="H1", source_channel="",
            destination_channel="").json()
        self.assertEqual(hardwareinjection_event['group'], "Test")
        self.assertEqual(hardwareinjection_event['pipeline'], "HardwareInjection")
        self.assertEqual(hardwareinjection_event['instruments'], "H1")

    def test_replace_event(self):
        time.sleep(self.SLEEP_TIME)
        graceid = self._eventId

        old_event = self._gracedb.event(graceid).json()
        self.assertEqual(old_event['group'], "Test")
        self.assertEqual(old_event['search'], "LowMass")
        self.assertEqual(float(old_event['gpstime']), 971609248.151741)

        replacementFile = os.path.join(self.TEST_DATA_DIR, "cbc-lm2.xml")

        response = self._gracedb.replaceEvent(graceid, replacementFile)
        self.assertEqual(response.status, 202)

        new_event = self._gracedb.event(graceid).json()
        self.assertEqual(new_event['group'], "Test")
        self.assertEqual(new_event['search'], "LowMass")
        self.assertEqual(float(new_event['gpstime']), 971609249.151741)

    def test_upload_binary(self):
        """
        Test workaround for Python bug
        http://bugs.python.org/issue11898
        Raises exception if workaround fails.
        """
        uploadFile = os.path.join(self.TEST_DATA_DIR, "upload.data.gz")
        r = self._gracedb.writeLog(self._eventId, "FILE UPLOAD", uploadFile)
        self.assertEqual(r.status, 201) # CREATED

    def test_unicode_param(self):
        """
        Test workaround for Python bug
        http://bugs.python.org/issue11898
        Raises exception if workaround fails.
        """
        uploadFile = os.path.join(self.TEST_DATA_DIR, "upload.data.gz")
        r = self._gracedb.writeLog(self._eventId, "FILE UPLOAD", uploadFile)
        self.assertEqual(r.status, 201) # CREATED

    def test_label_event(self):
        """Label an event"""
        r = self._gracedb.writeLabel(self._eventId, self._labelName)
        self.assertEqual(r.status, 201) # CREATED
        r = self._gracedb.labels(self._eventId, self._labelName)
        self.assertEqual(r.status, 200)
        label = r.json()
        self.assertEqual(self._labelName, label['name'])

    def test_remove_label_event(self):
        """Remove label added by test_label_event"""
        r = self._gracedb.removeLabel(self._eventId, self._labelName)
        self.assertEqual(r.status, 204)
        label_list = [l['name'] for l in
            self._gracedb.labels(self._eventId).json()['labels']]
        self.assertNotIn(self._labelName, label_list)

    def test_offline_param(self):
        """
        Tests whether offline parameter is transmitted to database
        properly or not. Creates an event with offline=True and checks it;
        also checks createdEvent, which uses offline=False.
        """
        self.assertEqual(self._createdEvent['offline'], self._offline)
        r = self._gracedb.createEvent("Test", "gstlal", self._eventFile,
            search="LowMass", offline=True)
        temp = r.json()
        self.assertEqual(temp['offline'], True)

    def test_logger(self):
        import logging
        import ligo.gracedb.rest
        import ligo.gracedb.logging

        logging.basicConfig()
        log = logging.getLogger('testing')
        log.propagate = False   # Don't write to console

        graceid = self._eventId

        handler = ligo.gracedb.logging.GraceDbLogHandler(self._gracedb, graceid)
        try:
            log.addHandler(handler)
            try:
                message = "Message is {0}".format(random.random())
                log.warning(message)
            finally:
                log.removeHandler(handler)
        finally:
            # Close the log handler in order to join with the writing thread.
            handler.close()

        event_logs = self._gracedb.logs(graceid).read()
        self.assertTrue(message.encode() in event_logs)



# -*- coding: utf-8 -*-
# Copyright (C) Brian Moe, Branson Stephens (2015)
#
# This file is part of gracedb
#
# gracedb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gracedb.  If not, see <http://www.gnu.org/licenses/>.
import unittest

from .test_events import TestEvents
from .test_labels import TestLabels
from .test_superevents import TestSuperevents
from .test_voevents import VOEventTestSuite

# List of TestCase classes to import tests from
test_classes = [TestEvents, TestSuperevents, TestLabels]

# List of test suites to add
test_suites = [VOEventTestSuite()]

# Define suite and loader
IntegrationTestSuite = unittest.TestSuite()
loader = unittest.TestLoader()

# Get suites from classes
for test_class in test_classes:
    suite = loader.loadTestsFromTestCase(test_class)
    test_suites.append(suite)

# Add custom test suites
for suite in test_suites:
    IntegrationTestSuite.addTest(suite)



# -*- coding: utf-8 -*-
# Copyright (C) Brian Moe, Branson Stephens (2015)
#
# This file is part of gracedb
#
# gracedb is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# It is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with gracedb.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function
import unittest
import os

from ligo.gracedb.rest import GraceDb


class TestGraceDb(unittest.TestCase):
    """Base class for gracedb-client integration tests"""

    @classmethod
    def setUpClass(cls):

        # Define useful variables
        # Test service URL
        TEST_SERVICE = os.environ.get('TEST_SERVICE',
            'https://gracedb-test.ligo.org/api/')

        # A sleep time in seconds used to slow down the event creation requests
        cls.SLEEP_TIME = 1

        # Data directory
        cls.TEST_DATA_DIR = os.environ.get('TEST_DATA_DIR',
            os.path.join(os.path.dirname(__file__), "data"))

        # Set up client
        cls._gracedb = GraceDb(TEST_SERVICE)
        print("Using service {0}".format(cls._gracedb._versioned_service_url))
